@@ -30,6 +30,11 @@
 #include "xst-ui.h"
 #include "xst-su.h"
 
+#if 0 /* Disabled */
+#include <config-archiver/archive.h>
+#include <config-archiver/location.h>
+#endif
+
 #include <gnome.h>
 #include <parser.h>
 #include <memory.h>
@@ -38,7 +43,13 @@
 #include <fcntl.h>
 #include <errno.h>
 
+#include <sys/types.h>
+#include <sys/wait.h>
+#include <signal.h>
+
 #define XST_DEBUG 1
+/* Define this if you want the frontend to run the backend under strace */
+/*#define XST_DEBUG_STRACE_BACKEND*/
 
 enum {
 	FILL_GUI,
@@ -49,11 +60,13 @@ enum {
 static enum {
 	ROOT_ACCESS_NONE,
 	ROOT_ACCESS_SIMULATED,
+	ROOT_ACCESS_SIMULATED_DISABLED,
 	ROOT_ACCESS_REAL
 } root_access = ROOT_ACCESS_NONE;
 
 static GtkObjectClass *parent_class;
 static gint xsttool_signals [LAST_SIGNAL] = { 0 };
+static const gchar *XST_TOOL_EOR = "\n<!-- XST: end of request -->\n";
 
 static gboolean platform_unsupported_cb   (XstTool *tool, XstReportLine *rline);
 static gboolean platform_add_supported_cb (XstTool *tool, XstReportLine *rline);
@@ -65,11 +78,12 @@ static void report_dispatch (XstTool *tool);
 static gint xst_tool_compare_platforms (XstPlatform *a, XstPlatform *b);
 
 static XstReportHookEntry common_report_hooks[] = {
+/*        key                 function                    type                      allow_repeat */
 	{ "end",              report_finished_cb,         XST_REPORT_HOOK_LOADSAVE, TRUE  },
 	{ "platform_list",    platform_add_supported_cb,  XST_REPORT_HOOK_LOAD,     TRUE  }, 
 	{ "platform_unsup",   platform_unsupported_cb,    XST_REPORT_HOOK_LOADSAVE, FALSE },
 	{ "platform_undet",   platform_unsupported_cb,    XST_REPORT_HOOK_LOADSAVE, FALSE },
-	{ "platform_success", platform_set_current_cb,    XST_REPORT_HOOK_LOAD,     FALSE }, 
+	{ "platform_success", platform_set_current_cb,    XST_REPORT_HOOK_SAVE,     FALSE }, 
 	{ 0,                  NULL,                       -1,                       FALSE }
 };
 
@@ -111,8 +125,7 @@ platform_unsupported_clicked_cb (GtkWidget *widget, gint ret, XstTool *tool)
 	if (ret == -1 || ret == 1)
 		exit (0);
 
-	/* Locate the selected platform and set it up as the current one. Prepare to
-	 * invoke the backend again, this time with the current platform specified. */
+	/* Locate the selected platform and set it up as the current one. */
 
 	if (tool->current_platform)
 		xst_platform_free (tool->current_platform);
@@ -124,55 +137,15 @@ platform_unsupported_clicked_cb (GtkWidget *widget, gint ret, XstTool *tool)
 
 	g_assert (tool->current_platform);
 
-	tool->report_dispatch_pending = FALSE;
-	tool->report_finished = TRUE;
-	tool->run_again = TRUE;
-
-	report_dispatch (tool);
-
 	gtk_signal_disconnect_by_func (GTK_OBJECT (tool->platform_dialog),
 				       platform_dialog_close_cb,
 				       tool);
-	
 }
 
 static gboolean
 platform_unsupported_cb (XstTool *tool, XstReportLine *rline)
 {
-	GSList *list;
-	XstPlatform *platform;
-	char *platform_text[1];
-
-	tool->report_dispatch_pending = TRUE;
-
-	/* Fill in the platform GtkCList */
-
-	gtk_clist_clear (GTK_CLIST (tool->platform_list));
-
-	for (list = tool->supported_platforms_list; list;
-	     list = g_slist_next (list))
-	{
-		platform = (XstPlatform *) list->data;
-
-		platform_text [0] = (char *) xst_platform_get_name (platform);
-		gtk_clist_append (GTK_CLIST (tool->platform_list), platform_text);
-	}
-
-	/* Prepare dialog and show it */
-
-	tool->platform_selected_row = -1;
-
-	gtk_signal_connect (GTK_OBJECT (tool->platform_list), "select-row",
-			    platform_list_select_row_cb, tool);
-	gtk_signal_connect (GTK_OBJECT (tool->platform_list), "unselect-row",
-			    platform_list_unselect_row_cb, tool);
-	gtk_signal_connect (GTK_OBJECT (tool->platform_dialog), "clicked",
-			    platform_unsupported_clicked_cb, tool);
-	gtk_signal_connect (GTK_OBJECT (tool->platform_dialog), "close",
-			    platform_dialog_close_cb, tool);
-
-	gtk_widget_set_sensitive (tool->platform_ok_button, FALSE);
-	gtk_widget_show (tool->platform_dialog);
+	tool->run_again = TRUE;
 
 	return TRUE;
 }
@@ -201,6 +174,7 @@ platform_set_current_cb (XstTool *tool, XstReportLine *rline)
 		xst_platform_free (tool->current_platform);
 
 	tool->current_platform = platform;
+	tool->run_again = FALSE;
 	return TRUE;
 }
 
@@ -209,6 +183,7 @@ platform_set_current_cb (XstTool *tool, XstReportLine *rline)
 static gboolean
 report_finished_cb (XstTool *tool, XstReportLine *rline)
 {
+	tool->report_dispatch_pending = FALSE;
 	tool->report_finished = TRUE;
 	return TRUE;
 }
@@ -444,9 +419,9 @@ report_progress_tick (gpointer data, gint fd, GdkInputCondition cond)
 
 				g_string_assign (tool->line, "");
 
-				if (!tool->report_dispatch_pending){
+				if (!tool->report_dispatch_pending) {
 					report_dispatch (tool);
-					if (tool->report_finished){
+					if (tool->report_finished) {
 						i++;
 						goto full_break;
 					}
@@ -492,12 +467,11 @@ report_window_close_cb (GtkWidget *window, gpointer data)
 }
 
 static void
-report_progress (XstTool *tool, int fd, const gchar *label)
+report_progress (XstTool *tool, const gchar *label)
 {
 #if 0
 	gint cb_id;
 #endif	
-
 	tool->timeout_done = FALSE;
 	tool->report_list_visible = FALSE;
 	tool->report_finished = FALSE;
@@ -506,18 +480,26 @@ report_progress (XstTool *tool, int fd, const gchar *label)
 	xst_tool_clear_supported_platforms (tool);
 	report_clear_lines (tool);
 
-	gtk_label_set_text (GTK_LABEL (tool->report_label), label);
 #if 0
 	set_arrow (tool, GTK_ARROW_DOWN);
 	gtk_progress_set_percentage (GTK_PROGRESS (tool->report_progress), 0.0);
 #endif
-	tool->input_id = gtk_input_add_full (fd, GDK_INPUT_READ, report_progress_tick,
-					     NULL, tool, NULL);
 
-	gtk_signal_connect_after (GTK_OBJECT (tool->report_window), "delete-event",
-				  GTK_SIGNAL_FUNC (report_window_close_cb), NULL);
-	gtk_widget_show (tool->report_window);
+	if (label) {
+		gtk_label_set_text (GTK_LABEL (tool->report_label), label);
+		gtk_signal_connect_after (GTK_OBJECT (tool->report_window), "delete-event",
+					  GTK_SIGNAL_FUNC (report_window_close_cb), NULL);
+		gtk_widget_show_all (tool->report_window);
+		while (gtk_events_pending ())
+		{
+			gtk_main_iteration ();
+			usleep (5000);
+		}
+	}
 	
+	tool->input_id = gtk_input_add_full (tool->backend_read_fd, GDK_INPUT_READ,
+					     report_progress_tick, NULL, tool, NULL);
+
 	gtk_main ();
 
 	if (tool->input_id)
@@ -541,27 +523,351 @@ report_progress (XstTool *tool, int fd, const gchar *label)
 #endif
 }
 
+static void
+xst_tool_run_platform_dialog (XstTool *tool)
+{
+	GSList *list;
+	XstPlatform *platform;
+	char *platform_text[1];
+
+	/* Fill in the platform GtkCList */
+
+	gtk_clist_clear (GTK_CLIST (tool->platform_list));
+
+	for (list = tool->supported_platforms_list; list;
+	     list = g_slist_next (list))
+	{
+		platform = (XstPlatform *) list->data;
+
+		platform_text [0] = (char *) xst_platform_get_name (platform);
+		gtk_clist_append (GTK_CLIST (tool->platform_list), platform_text);
+	}
+
+	/* Prepare dialog and show it */
+
+	tool->platform_selected_row = -1;
+
+	gtk_signal_connect (GTK_OBJECT (tool->platform_list), "select-row",
+			    platform_list_select_row_cb, tool);
+	gtk_signal_connect (GTK_OBJECT (tool->platform_list), "unselect-row",
+			    platform_list_unselect_row_cb, tool);
+	gtk_signal_connect (GTK_OBJECT (tool->platform_dialog), "clicked",
+			    platform_unsupported_clicked_cb, tool);
+	gtk_signal_connect (GTK_OBJECT (tool->platform_dialog), "close",
+			    platform_dialog_close_cb, tool);
+
+	gtk_widget_set_sensitive (tool->platform_ok_button, FALSE);
+
+	gnome_dialog_run (GNOME_DIALOG (tool->platform_dialog));
+}
+
+static void
+xst_tool_process_startup (XstTool *tool)
+{
+	/* let's process those startup reports. */
+	tool->report_hook_type = XST_REPORT_HOOK_LOAD;
+	report_progress (tool, NULL);
+	if (tool->run_again) {
+		xst_tool_run_platform_dialog (tool);
+		g_assert (tool->current_platform);
+		
+		if (root_access == ROOT_ACCESS_SIMULATED)
+			root_access = ROOT_ACCESS_SIMULATED_DISABLED;
+		
+		xst_tool_run_set_directive (tool, NULL, NULL, "platform_set",
+					    tool->current_platform->name, NULL);
+
+		if (root_access == ROOT_ACCESS_SIMULATED_DISABLED)
+			root_access = ROOT_ACCESS_SIMULATED;
+	}
+}
+
+static void
+xst_tool_init_backend (XstTool *tool)
+{
+	int fd_read [2], fd_write [2];
+
+	g_return_if_fail (tool != NULL);
+	g_return_if_fail (XST_IS_TOOL (tool));
+	g_return_if_fail (tool->script_path != NULL);
+	g_return_if_fail (tool->backend_pid < 0);
+	
+	pipe (fd_read);
+	pipe (fd_write);
+
+	tool->backend_pid = fork ();
+	if (tool->backend_pid < 0) {
+		g_warning ("Unable to fork.");
+		tool->backend_pid = -1;
+		return;
+	} else if (tool->backend_pid) {
+		/* Parent */
+
+		close (fd_read  [1]); /* Close writing end of read pipe */
+		close (fd_write [0]); /* Close reading end of write pipe */
+
+		tool->backend_read_fd  = fd_read  [0];
+		tool->backend_write_fd = fd_write [1];
+
+		fcntl (tool->backend_read_fd, F_SETFL, 0);  /* Let's block */
+
+		xst_tool_process_startup (tool);
+	} else {
+		/* Child */
+
+		/* For the child, things are upside-down */
+		close (fd_read  [0]); /* Close reading end of read pipe */
+		close (fd_write [1]); /* Close writing end of write pipe */
+
+		dup2 (fd_read  [1], STDOUT_FILENO);
+		dup2 (fd_write [0],  STDIN_FILENO);
+
+		if (tool->current_platform)
+			execl (tool->script_path, tool->script_path,
+			       "--progress", "--report", "--platform",
+			       xst_platform_get_name (tool->current_platform), NULL);
+		else
+#ifndef XST_DEBUG_STRACE_BACKEND
+			execl (tool->script_path, tool->script_path, "--progress",
+			       "--report", NULL);
+#else		
+		execl ("/usr/bin/strace", "/usr/bin/strace", "-f", tool->script_path, "--progress",
+		       "--report", NULL);
+#endif					
+		
+		g_error ("Unable to run backend: %s", tool->script_path);
+	}
+}
+
+static void
+xst_tool_kill_backend (XstTool *tool, gpointer data)
+{
+	g_return_if_fail (tool != NULL);
+	g_return_if_fail (XST_IS_TOOL (tool));
+	
+	/* The backend was never called. No problem! */
+	/* For further reference check http://www.xs4all.nl/~pjbrink/apekool/050/045-en.html */
+	if (tool->backend_pid < 0)
+		return;
+
+	if (root_access == ROOT_ACCESS_SIMULATED)
+		root_access = ROOT_ACCESS_SIMULATED_DISABLED;
+		
+	xst_tool_run_set_directive (tool, NULL, _("Closing tool."), "end", NULL);
+
+	if (root_access == ROOT_ACCESS_SIMULATED_DISABLED)
+		root_access = ROOT_ACCESS_SIMULATED;
+	
+	waitpid (tool->backend_pid, NULL, 0);
+}
+
+static gboolean
+xst_tool_end_of_request (GString *string)
+{
+	gchar *str;
+	gint len, eorlen, i;
+
+	str = string->str;
+	len = strlen (str);
+	eorlen = strlen (XST_TOOL_EOR);
+	if (len < eorlen)
+		return FALSE;
+
+	for (i = 0; i < eorlen; i++)
+		if (XST_TOOL_EOR[eorlen - i - 1] != str[len - i - 1])
+			return FALSE;
+
+	g_string_truncate (string, len - eorlen + 1);
+
+	return TRUE;
+}
+
+static xmlDoc *
+xst_tool_read_xml_from_backend (XstTool *tool)
+{
+	char buffer [4096];
+	int t;
+	xmlDoc *xml;
+
+	if (!tool->xml_document)
+		return NULL;
+	
+	fcntl (tool->backend_read_fd, F_SETFL, O_NONBLOCK);
+
+	while (! xst_tool_end_of_request (tool->xml_document)) {
+		while (gtk_events_pending ())
+			gtk_main_iteration ();
+
+		t = read (tool->backend_read_fd, buffer, sizeof (buffer) - 1);
+
+		if (t == 0)
+			break;
+		if (t == -1)
+			t = 0;
+
+		buffer [t] = 0;
+		g_string_append (tool->xml_document, buffer);
+	}
+	
+	fcntl (tool->backend_read_fd, F_SETFL, 0);
+
+	if (tool->xml_document->str[0] == '<') {
+		xml = xmlParseDoc (tool->xml_document->str);
+	} else {
+		xml = NULL;
+	}
+
+	g_string_free (tool->xml_document, TRUE);
+	tool->xml_document = NULL;
+
+	return xml;
+}
+
+static void
+xst_tool_send_directive (XstTool *tool, const gchar *directive, va_list ap)
+{
+	GString *directive_line;
+	gchar *arg;
+	FILE *f;
+
+	directive_line = g_string_new (directive);
+	while ((arg = va_arg (ap, char *)) != NULL) {
+		g_string_append (directive_line, "::");
+		g_string_append (directive_line, arg);
+	}
+	va_end (ap);
+	g_string_append_c (directive_line, '\n');
+	
+	if (tool->backend_pid < 0)
+		xst_tool_init_backend (tool);
+
+	f = fdopen (dup (tool->backend_write_fd), "w");
+	fprintf (f, directive_line->str);
+	fclose (f);
+
+	g_string_free (directive_line, TRUE);
+}
+
+xmlDoc *
+xst_tool_run_get_directive (XstTool *tool, const gchar *report_sign, const gchar *directive, ...)
+{
+	va_list ap;
+	xmlDoc *xml;
+	const gchar *directive_ptr;
+	
+	g_return_val_if_fail (tool != NULL, NULL);
+	g_return_val_if_fail (XST_IS_TOOL (tool), NULL);
+
+	directive_ptr = directive;
+	va_start (ap, directive);
+	xst_tool_send_directive (tool, directive_ptr, ap);
+
+	/* FIXME: Instead of doing the following, we should pass around a value describing
+	 * tool I/O mode (as opposed to a string to report_progress ()). */
+	tool->report_hook_type = XST_REPORT_HOOK_LOAD;
+
+	xmlSubstituteEntitiesDefault (TRUE);
+
+	/* LibXML support for parsing from memory is good, but parsing from
+	 * opened filehandles is not supported unless you write your own feed
+	 * mechanism. Let's just load it all into memory, then. Also, refusing
+	 * enormous documents can be considered a plus. </dystopic> */
+	
+	if (tool->xml_document == NULL)
+		tool->xml_document = g_string_new ("");
+	
+	if (location_id == NULL)
+		report_progress (tool, _(report_sign));
+
+	xml = xst_tool_read_xml_from_backend (tool);
+	
+	return xml;
+}
+
+xmlDoc *
+xst_tool_run_set_directive (XstTool *tool, xmlDoc *xml,
+			    const gchar *report_sign, const gchar *directive, ...)
+{
+	va_list ap;
+	FILE *f;
+	xmlDoc *xml_out;
+	const gchar *directive_ptr;
+	
+	/* don't actually run if we are just pretending */
+	if (root_access == ROOT_ACCESS_SIMULATED) {
+		g_warning (_("Skipping set directive..."));
+		return NULL;
+	}
+
+	g_return_val_if_fail (tool != NULL, NULL);
+	g_return_val_if_fail (XST_IS_TOOL (tool), NULL);
+
+	directive_ptr = directive;
+	va_start (ap, directive);
+	xst_tool_send_directive (tool, directive_ptr, ap);
+
+	if (xml) {
+		f = fdopen (dup (tool->backend_write_fd), "w");
+		xmlDocDump (f, xml);
+		fprintf (f, XST_TOOL_EOR);
+		fclose (f);
+	}
+
+	/* FIXME: Instead of doing the following, we should pass around a value describing
+	 * tool I/O mode (as opposed to a string to report_progress ()). */
+	tool->report_hook_type = XST_REPORT_HOOK_SAVE;
+
+	if (location_id == NULL)
+		report_progress (tool, report_sign? _(report_sign): NULL);
+
+	/* This is tipicaly to just read the end of request string,
+	   but a set directive may return some XML too. */
+	xml_out = xst_tool_read_xml_from_backend (tool);
+	return xml_out;
+}
+
+
+gboolean
+xst_tool_load (XstTool *tool)
+{
+	g_return_val_if_fail (tool != NULL, FALSE);
+	g_return_val_if_fail (XST_IS_TOOL (tool), FALSE);	
+	g_return_val_if_fail (tool->script_path, FALSE);
+  
+	if (tool->config) {
+		xmlFreeDoc (tool->config);
+		tool->config = NULL;
+	}
+
+	if (tool->run_again)
+		return TRUE;
+
+	tool->config = xst_tool_run_get_directive (tool, _("Scanning your system configuration."),
+						   "get", NULL);
+	
+	if (tool->config)
+		gtk_signal_emit (GTK_OBJECT (tool), xsttool_signals[FILL_GUI]);
+
+	return tool->config != NULL;
+}
+
 gboolean
 xst_tool_save (XstTool *tool)
 {
-	FILE *f;
-	int fd_xml [2], fd_report [2];
-	int t;
+#if 0 /* Disabled */
+	Archive *archive;
+	Location *location;
+#endif
+	gchar *backend_id;
 
 	g_return_val_if_fail (tool != NULL, FALSE);
 	g_return_val_if_fail (XST_IS_TOOL (tool), FALSE);
-	g_return_val_if_fail (tool->script_path, FALSE);
-
 	g_return_val_if_fail (root_access != ROOT_ACCESS_NONE, FALSE);
 
 	xst_dialog_freeze_visible (tool->main_dialog);
 
 	gtk_signal_emit (GTK_OBJECT (tool), xsttool_signals[FILL_XML]);
 
-	/* FIXME: Instead of doing the following, we should pass around a value describing
-	 * tool I/O mode (as opposed to a string to report_progress ()). */
-	tool->report_hook_type = XST_REPORT_HOOK_SAVE;
-
 #ifdef XST_DEBUG
 	/* don't actually save if we are just pretending */
 	if (root_access == ROOT_ACCESS_SIMULATED) {
@@ -570,51 +876,35 @@ xst_tool_save (XstTool *tool)
 	}
 #endif
 
-	pipe (fd_xml);
-	pipe (fd_report);
+#if 0 /* Disabled */
+	/* Archive data with the archiver */
+	archive = ARCHIVE (archive_load (TRUE));
 
-	t = fork ();
-	if (t < 0) {
-		g_error ("Unable to fork.");
-	} else if (t) {
-		/* Parent */
+	if (location_id == NULL)
+		location = archive_get_current_location (archive);
+	else
+		location = archive_get_location (archive, location_id);
 
-		close (fd_xml [0]);	/* Close reading end of XML pipe */
-		close (fd_report [1]);  /* Close writing end of report pipe */
+	backend_id = strrchr (tool->script_path, '/');
 
-		f = fdopen (fd_xml [1], "w");
+	if (backend_id != NULL)
+		backend_id++;
+	else
+		backend_id = tool->script_path;
 
-		xmlDocDump (f, tool->config);
-		
-		fclose (f);
-		
-		report_progress (tool, fd_report [0], _("Updating your system configuration."));
-		close (fd_report [0]);
+	location_store_xml (location, backend_id, tool->config, STORE_MASK_PREVIOUS);
 
-	} else {
-		gchar *loc_arg;
+	archive_close (archive);
+#endif
 
-		/* Child */
-
-		close (fd_xml [1]);	/* Close writing end of XML pipe */
-		close (fd_report [0]);  /* Close reading end of report pipe */
-		dup2 (fd_xml [0], STDIN_FILENO);
-		dup2 (fd_report [1], STDOUT_FILENO);
-
-		loc_arg = (location_id == NULL) ? NULL : "--location";
-
-		if (tool->current_platform)
-			execl (tool->script_path, tool->script_path,
-			       "--set", "--progress", "--report", "--platform",
-			       xst_platform_get_name (tool->current_platform),
-			       loc_arg, location_id, NULL);
-		else
-			execl (tool->script_path, tool->script_path, "--set", "--progress",
-			       "--report", loc_arg, location_id, NULL);
-
-		g_error ("Unable to run backend: %s", tool->script_path);
+	if (location_id != NULL) {
+		xst_dialog_thaw_visible (tool->main_dialog);
+		return TRUE;
 	}
 
+	xst_tool_run_set_directive (tool, tool->config, _("Updating your system configuration."),
+				    "set", NULL);
+
 	xst_dialog_thaw_visible (tool->main_dialog);
 	return TRUE;  /* FIXME: Determine if it really worked. */
 }
@@ -625,112 +915,17 @@ xst_tool_save_cb (GtkWidget *w, XstTool *tool)
 	xst_tool_save (tool);
 }
 
-gboolean
-xst_tool_load (XstTool *tool)
-{
-	int fd [2];
-	int t;
-
-	g_return_val_if_fail (tool != NULL, FALSE);
-	g_return_val_if_fail (XST_IS_TOOL (tool), FALSE);	
-	g_return_val_if_fail (tool->script_path, FALSE);
-  
-	/* FIXME: Instead of doing the following, we should pass around a value describing
-	 * tool I/O mode (as opposed to a string to report_progress ()). */
-	tool->report_hook_type = XST_REPORT_HOOK_LOAD;
-
-	xmlSubstituteEntitiesDefault (TRUE);
-
-	if (tool->config) {
-		xmlFreeDoc (tool->config);
-		tool->config = NULL;
-	}
-
-	if (location_id == NULL) {
-		pipe (fd);
-		t = fork ();
-	} else {
-		fd[0] = STDIN_FILENO;
-		fd[1] = -1;
-		t = 1;
-	}
-
-	if (t < 0) {
-		g_error ("Unable to fork.");
-	} else if (t) {
-		char buffer [2048];
-		
-		/* Parent */
-
-		close (fd [1]);	/* Close writing end */
-
-		/* LibXML support for parsing from memory is good, but parsing from
-		 * opened filehandles is not supported unless you write your own feed
-		 * mechanism. Let's just load it all into memory, then. Also, refusing
-		 * enormous documents can be considered a plus. </dystopic> */
-
-		if (location_id == NULL)
-			report_progress (tool, fd [0], _("Scanning your system configuration."));
-
-		if (tool->run_again)
-			return TRUE;
-
-		if (tool->xml_document == NULL)
-			tool->xml_document = g_string_new ("");
-		
-		fcntl(fd [0], F_SETFL, 0);  /* Let's block */
-
-		while ((t = read (fd [0], buffer, sizeof (buffer)-1)) != 0){
-			buffer [t] = 0;
-			g_string_append (tool->xml_document, buffer);
-		}
-		
-		tool->config = xmlParseDoc (tool->xml_document->str);
-		g_string_free (tool->xml_document, TRUE);
-		tool->xml_document = NULL;
-		close (fd [0]);
-	} else {
-		/* Child */
-
-		close (fd [0]);	/* Close reading end */
-		dup2 (fd [1], STDOUT_FILENO);
-
-		if (tool->current_platform)
-			execl (tool->script_path, tool->script_path,
-			       "--get", "--progress", "--report", "--platform",
-			       xst_platform_get_name (tool->current_platform),
-			       NULL);
-		else
-			execl (tool->script_path, tool->script_path, "--get", "--progress",
-			       "--report", NULL);
-
-		g_error ("Unable to run backend %s: %s", tool->script_path,
-			 g_strerror (errno));
-	}
-	
-	if (tool->config)
-		gtk_signal_emit (GTK_OBJECT (tool), xsttool_signals[FILL_GUI]);
-
-	return tool->config != NULL;
-}
-
 void
 xst_tool_load_try (XstTool *tool)
 {
 	GtkWidget *d;
 
-	do
-	{
-		tool->run_again = FALSE;
-
-		if (!xst_tool_load (tool)) {
-			d = gnome_ok_dialog (_("There was an error running the backend script,\n"
-					       "and the configuration could not be loaded."));
-			gnome_dialog_run_and_close (GNOME_DIALOG (d));
-			exit (1);
-		}
+	if (!xst_tool_load (tool)) {
+		d = gnome_ok_dialog (_("There was an error running the backend script,\n"
+				       "and the configuration could not be loaded."));
+		gnome_dialog_run_and_close (GNOME_DIALOG (d));
+		exit (0);
 	}
-	while (tool->run_again);
 }
 
 void
@@ -914,6 +1109,9 @@ xst_tool_construct (XstTool *tool, const char *name, const char *title)
 
 	tool->report_hook_list = NULL;
 	xst_tool_add_report_hooks (tool, common_report_hooks);
+
+	tool->backend_pid = tool->backend_read_fd = tool->backend_write_fd = -1;
+	xst_tool_set_close_func (tool, xst_tool_kill_backend, NULL);
 }
 
 XstTool *
