@@ -6,6 +6,7 @@
 
 GtkWidget *xst_ui_list_get_list_item_by_name      (GtkList *list, const gchar *label);
 void       xst_ui_combo_remove_by_label           (GtkCombo *combo, const gchar *label);
+void       xst_ui_image_set_pix                   (GtkWidget *widget, gchar *filename);
 GtkWidget *xst_ui_create_image_widget             (gchar *name, gchar *string1, gchar *string2,
 						   gint int1, gint int2);
 gint       xst_ui_option_menu_get_selected_row    (GtkOptionMenu *option_menu);
@@ -21,5 +22,8 @@ gboolean   xst_ui_clist_get_checkmark             (GtkCList *clist, gint row, gi
 void       xst_ui_ctree_set_checkmark             (GtkCTree *ctree, GtkCTreeNode *node,
 						   gint column, gboolean state);
 gboolean   xst_ui_ctree_get_checkmark             (GtkCTree *ctree, GtkCTreeNode *node, gint column);
+gchar     *xst_ui_load_etspec                     (const gchar *common_path, const gchar *name);
+void       xst_ui_entry_set_text                  (void *entry, const gchar *str);
+gboolean   xst_ui_logout_dialog                   (const gchar *message);
 
 #endif /* XST_UI_H */
