@@ -21,10 +21,19 @@
  * Authors: Tambet Ingo <tambet@ximian.com> and Arturo Espinosa <arturo@ximian.com>.
  */
 
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <ctype.h>
 #include <gnome.h>
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+#include <errno.h>
+
 #include "callbacks.h"
 #include "user_group.h"
 #include "e-table.h"
@@ -191,7 +200,7 @@ user_filter (xmlNodePtr node)
 }
 
 gboolean
-check_node_complexity (xmlNodePtr node)
+check_node_visibility (xmlNodePtr node)
 {
 	xmlNodePtr db_node;
 	gchar *field, *content;
@@ -333,7 +342,7 @@ check_user_login (xmlNodePtr node, const gchar *login)
 
 	/* if !exist. */
 	else if (node_exists (node, "login", login))
-		buf = g_strdup (_("Username already exists."));
+		buf = g_strdup_printf (_("Username \"%s\" already exists.\n\nPlease select a different Username"), login);
 
 	return buf;
 }
@@ -368,8 +377,11 @@ user_account_check_comment (UserAccount *account)
 	comment = g_strjoinv (NULL, account->comment);
 	
 	for (i = 0; i < strlen (comment); i++) {
-		if (!isspace (comment[i]) && !isalnum (comment[i])) {
-			buf = g_strdup (N_("Invalid comment"));
+		if (iscntrl (comment[i]) || comment[i] == ',' ||
+		    comment[i] == '=' || comment[i] == ':') {
+
+			buf = g_strdup_printf (N_("Invalid character '%c' in comment."),
+						  comment[i]);
 			break;
 		}
 	}
@@ -418,7 +430,8 @@ user_account_check_home (UserAccount *account)
 {
 	gchar *home = account->home;
 	xmlNodePtr node = account->node;
-
+	struct stat s;
+	
 	if (!home || (strlen (home) < 1))
 		return g_strdup (N_("Home directory must not be empty."));
 
@@ -428,6 +441,16 @@ user_account_check_home (UserAccount *account)
 	if (!check_user_root (node, "home", home))
 		return g_strdup (N_("root user shouldn't be modified."));
 
+	if (stat (home, &s))
+	{
+		switch (errno) {
+		case ENOTDIR: return _("Part of the path to the home directory is a file.");         
+		case ELOOP:   return _("There is a loop in the path to the home directory.");        
+		case ENOMEM:  return _("Not enough memory to check the home directory.");            
+		case ENAMETOOLONG: return _("The path to the home directory is too long.");          
+		}
+	}
+
 	return NULL;
 }
 
@@ -639,7 +662,7 @@ get_user_list (gchar *field, xmlNodePtr group_node)
 	     u = xst_xml_element_find_next (u, "user"))
 	{
 
-		if (check_node_complexity (u))
+		if (check_node_visibility (u))
 			list = g_list_prepend (list, xst_xml_get_child_content (u, field));
 	}
 
@@ -1292,6 +1315,8 @@ user_account_save (UserAccount *account)
 	gchar *buf;
 	xmlNodePtr node = account->node;
 
+	account->modified = FALSE;
+
 	if (account->new)
 		node = account->node = user_add_blank_xml (account->node);
 	
@@ -1348,6 +1373,91 @@ user_account_check (UserAccount *account)
 	return NULL;
 }
 
+static GList *
+user_account_check_home_warnings (UserAccount *account, GList *warnings)
+{
+	gchar *home;
+	struct stat s;
+
+	home = parse_home (account);
+
+	if (stat (home, &s))
+	{
+		gchar *text = NULL;
+		
+		switch (errno) {
+		case ENOENT:
+			if (!account->new)
+				text = _("The home directory does not exist, or its path is invalid.");
+			break;
+		case EACCES:
+			if (!account->new)
+				text = _("Couldn't get access to the home directory.");
+			break;
+		default:
+			if (!account->new)
+				/* We shouldn't fall here: the other cases are in the error checks. */
+				text = _("There was an error trying to access the home directory.");
+		}
+
+		if (text)
+			warnings = g_list_append (warnings, text);
+	} else {
+		if (account->new) {
+			warnings = g_list_append
+				(warnings, _("The home directory already exists. "
+					     "This will break the new user's permissions."));
+		} else {
+			if (!S_ISDIR (s.st_mode))
+				warnings = g_list_append
+					(warnings, _("The home directory path exists, "
+						     "but it is not a directory."));
+			if (account->uid && (s.st_uid != atoi (account->uid)))
+				warnings = g_list_append
+					(warnings, _("The home directory is not owned by the user."));
+			else {
+				if (!s.st_mode & S_IRUSR)
+					warnings = g_list_append
+						(warnings, _("The user doesn't have permission to "
+							     "read from the home directory."));
+				if (!s.st_mode & S_IWUSR)
+					warnings = g_list_append
+						(warnings, _("The user doesn't have permission to "
+							     "write in the home directory."));
+				if (!s.st_mode & S_IRUSR)
+					warnings = g_list_append
+						(warnings, _("The user doesn't have permission to "
+							     "use (``execute'') the home directory."));
+			}
+		}
+	}
+
+	g_free (home);
+		
+	return warnings;
+}
+
+GList *
+user_account_check_warnings (UserAccount *account)
+{
+	GList *warnings = NULL;
+
+	warnings = user_account_check_home_warnings (account, warnings);
+
+	return warnings;
+}
+
+void
+user_account_destroy_warnings (GList *warnings)
+{
+	GList *l, *l2;
+
+	for (l2 = l = warnings; l2; l = l2) {
+		l2 = l->next;
+		g_list_remove_link (warnings, l);
+	}
+}
+
 void
 user_account_destroy (UserAccount *account)
 {
